/// <reference types="node" />
import { TableInterface, ControllBarInterface, ControllDragging, ControllChangeSize, ControllOptions, ControllDraggingHeader } from '../types';
import { EventEmitter2 } from 'eventemitter2';
import { EditorInterface, NodeInterface } from '@aomao/engine';
declare class ControllBar extends EventEmitter2 implements ControllBarInterface {
    private editor;
    private table;
    private readonly COL_MIN_WIDTH;
    private readonly ROW_MIN_HEIGHT;
    tableRoot?: NodeInterface;
    colsHeader?: NodeInterface;
    rowsHeader?: NodeInterface;
    tableHeader?: NodeInterface;
    menuBar?: NodeInterface;
    dragging?: ControllDragging;
    draggingHeader?: ControllDraggingHeader;
    changeSize?: ControllChangeSize;
    viewport?: NodeInterface;
    placeholder?: NodeInterface;
    contextVisible: boolean;
    rowDeleteButton?: NodeInterface;
    colDeleteButton?: NodeInterface;
    colAddButton?: NodeInterface;
    colAddAlign?: 'right' | 'left';
    colAddButtonSplit?: NodeInterface;
    moveColIndex: number;
    hideColAddButtonTimeount?: NodeJS.Timeout;
    rowAddButton?: NodeInterface;
    rowAddAlign?: 'up' | 'down';
    rowAddButtonSplit?: NodeInterface;
    moveRowIndex: number;
    hideRowAddButtonTimeount?: NodeJS.Timeout;
    constructor(editor: EditorInterface, table: TableInterface, options: ControllOptions);
    init(): void;
    renderRowBars(start?: number, end?: number): void;
    renderColBars(): void;
    /**
     * 绑定事件
     */
    bindEvents(): void;
    /**
     * 在表格上单击
     * @param event
     */
    onTableMouseDown(event: MouseEvent): void;
    /**
     * 鼠标在列表头上移动
     * @param event
     */
    onMouseMoveColsHeader(event: MouseEvent): void;
    /**
     * 鼠标在行表头上移动
     * @param event
     * @returns
     */
    onMouseMoveRowsHeader(event: MouseEvent): void;
    /**
     * 鼠标在列头部按下
     * @param event 事件
     * @returns
     */
    onMouseDownColsHeader: (event: MouseEvent | TouchEvent) => void;
    /**
     * 鼠标在行头部按下
     * @param event 事件
     * @returns
     */
    onMouseDownRowsHeader: (event: MouseEvent | TouchEvent) => void;
    /**
     * 鼠标在列头部单击
     * @param event 事件
     */
    onClickColsHeader: (event: MouseEvent) => void;
    /**
     * 鼠标在行头部单击
     * @param event 事件
     */
    onClickRowsHeader: (event: MouseEvent) => void;
    /**
     * 鼠标在表格左上角头部单击
     * @param event 事件
     */
    onClickTableHeader: (event: MouseEvent) => void;
    /**
     * 激活表头状态
     * @returns
     */
    activeHeader(): void;
    /**
     * 清楚表头活动状态
     */
    clearActiveStatus(): void;
    /**
     * 刷新控制UI
     */
    refresh(): void;
    /**
     * 开始改变列宽度
     * @param col 列节点
     * @param event 事件
     */
    startChangeCol(trigger: NodeInterface, event: MouseEvent | TouchEvent): void;
    /**
     * 开始改变行高度
     * @param col 列节点
     * @param event 事件
     */
    startChangeRow(trigger: NodeInterface, event: MouseEvent | TouchEvent): void;
    /**
     * 绑定改变大小事件
     */
    bindChangeSizeEvent(): void;
    /**
     * 移除绑定改变不大小事件
     */
    unbindChangeSizeEvent(): void;
    onChangeSize: (event: MouseEvent | TouchEvent) => void;
    /**
     * 列宽度改变
     * @param event 事件
     * @returns
     */
    onChangeColWidth(event: MouseEvent | TouchEvent): void;
    onChangeRowHeight(event: MouseEvent | TouchEvent): void;
    renderColSplitBars(col: NodeInterface, trigger: NodeInterface): void;
    renderRowSplitBars(row: NodeInterface, trigger: NodeInterface): void;
    onChangeSizeEnd: (event: MouseEvent | TouchEvent) => void;
    onDragStartColsHeader: (event: DragEvent) => void;
    onDragStartRowsHeader: (event: DragEvent) => void;
    bindDragColEvent(): void;
    unbindDragColEvent(): void;
    bindDragRowEvent(): void;
    unbindDragRowEvent(): void;
    showPlaceHolder(dropIndex: number, isNext?: boolean): void;
    onDragCol: (event: DragEvent) => void;
    onDragColEnd: () => void;
    onDragRow: (event: DragEvent) => void;
    onDragRowEnd: () => void;
    removeRow(index: number): void;
    removeCol(index: number): void;
    showContextMenu(event: MouseEvent): void;
    onMenuInputMousedown: (event: MouseEvent) => void;
    hideContextMenu(): void;
    getMenuDisabled(action: string): boolean;
    handleClickMenu(event: MouseEvent): void;
    handleTriggerMenu(menu: NodeInterface): void;
    handleHoverMenu(event: MouseEvent): void;
    hideHighlight(event: MouseEvent): void;
    handleHighlightRow: () => void;
    handleHighlightCol: () => void;
    handleHighlightTable: () => void;
    drawBackgroundColor(color?: string): void;
    setAlign(align?: 'top' | 'middle' | 'bottom'): void;
    destroy(): void;
}
export default ControllBar;
