import { HelperInterface, TableModel, TableModelCol, TableModelEmptyCol } from '../types';
import { NodeInterface, EditorInterface } from '@aomao/engine';
declare class Helper implements HelperInterface {
    #private;
    private clipboard?;
    constructor(editor: EditorInterface);
    isEmptyModelCol(model: TableModelCol | TableModelEmptyCol): model is TableModelEmptyCol;
    /**
     * 获取表格数据模型
     * @param table
     * @returns
     */
    getTableModel(table: NodeInterface): TableModel;
    /**
     * 修复表格，补全丢失的单元格
     * @param table
     * @returns
     */
    normalize(table: NodeInterface): NodeInterface;
    /**
     * 过滤 table 中的首行空tr, 当表格尾部有空白单元格时，网页拷贝时头部会莫名其妙的出现一个空的Tr
     * @param {nodeModel} table 传入的table
     */
    trimStartTr(table: NodeInterface): void;
    fixNumberTr(table: NodeInterface): void;
    fixDragEvent(event: DragEvent): void;
    /**
     * 从源节点复制样式到目标节点
     * @param from 源节点
     * @param to 目标节点
     */
    copyCss(from: NodeInterface | Node, to: NodeInterface | Node): void;
    /**
     * 从源节点复制样式和内容到目标节点
     * @param from 源节点
     * @param to 目标节点
     */
    copyTo(from: NodeInterface | Node, to: NodeInterface | Node): void;
    /**
     * 复制html
     * @param html HTML
     */
    copyHTML(html: string): void;
    /**
     * 获取复制的数据
     * @returns
     */
    getCopyData(): {
        html: string;
        text: string;
    } | undefined;
    trimBlankSpan(node: NodeInterface): NodeInterface;
    /**
     * table 结构标准化，补齐丢掉的单元格和行
     * 场景1. number 拷贝过来的 html 中，如果这一行没有单元格，就会省掉 tr，渲染的时候会有问题
     * 场景2. 从网页中鼠标随意选取表格中的一部分，会丢掉没有选中的单元格，需要补齐单元格
     * @param {NodeInterface} table 表格 Dom
     * @return {NodeInterface} 修复过的 table dom
     */
    normalizeTable(table: NodeInterface): NodeInterface;
}
export default Helper;
