import { TableInterface, TableModel, TableSelectionInterface, TableSelectionArea, TableSelectionDragging, TableModelCol, TableModelEmptyCol } from '../types';
import { EventEmitter2 } from 'eventemitter2';
import { EditorInterface, NodeInterface } from '@aomao/engine';
declare class TableSelection extends EventEmitter2 implements TableSelectionInterface {
    private editor;
    private table;
    tableRoot?: NodeInterface;
    colsHeader?: NodeInterface;
    rowsHeader?: NodeInterface;
    tableHeader?: NodeInterface;
    tableModel?: TableModel;
    selectArea?: TableSelectionArea;
    selectRange?: {
        type: 'left' | 'right' | 'top' | 'bottom';
        startOffset: number;
        endOffset: number;
    };
    dragging?: TableSelectionDragging;
    isShift: boolean;
    prevMouseDownTd?: NodeInterface;
    prevOverTd?: NodeInterface;
    highlight?: NodeInterface;
    beginAllRow?: number;
    beginAllCol?: number;
    constructor(editor: EditorInterface, table: TableInterface);
    init(): void;
    render(action: string): void;
    renderBorder(): void;
    bindEvents(): void;
    unbindEvents(): void;
    refreshModel(): void;
    each(fn: (cell: TableModelCol | TableModelEmptyCol, row: number, col: number) => void, reverse?: boolean): void;
    getCellPoint(td: NodeInterface): number[];
    getCellIndex(row: number, col: number): number;
    getSingleCell(): NodeInterface | null;
    getSingleCellPoint(): number[];
    getSelectArea(): TableSelectionArea;
    selectCol(begin: number, end?: number): void;
    selectRow(begin: number, end?: number): void;
    selectCell(begin: NodeInterface, end: NodeInterface): void;
    clearSelect(): void;
    select(begin: {
        row: number;
        col: number;
    }, end: {
        row: number;
        col: number;
    }): void;
    focusCell(cell: NodeInterface | Node, start?: boolean): void;
    selectCellRange(cell: NodeInterface | Node): void;
    onTdMouseDown: (event: MouseEvent | TouchEvent) => void;
    addDragEvent(): void;
    removeDragEvent: () => void;
    onDragMove: (event: MouseEvent | TouchEvent) => void;
    onShiftKeydown: (event: KeyboardEvent) => void;
    onKeydown: (event: KeyboardEvent) => void;
    onShiftKeyup: (event: KeyboardEvent) => void;
    selectLeft(event: KeyboardEvent, td: NodeInterface): void;
    selectRigth(event: KeyboardEvent, td: NodeInterface): void;
    selectUp(event: KeyboardEvent, td: NodeInterface): void;
    selectDown(event: KeyboardEvent, td: NodeInterface): void;
    getSelectionHtml(all?: boolean): string | null;
    hasMergeCell(): boolean;
    isRowSelected(): boolean;
    isColSelected(): boolean;
    isTableSelected(): boolean;
    showHighlight(area: TableSelectionArea): void;
    hideHighlight(): void;
    destroy(): void;
}
export default TableSelection;
